@@ -1,3 +1,4 @@
+pub(crate) use self::pure::pure_optimizer;
 use self::{
     drop_console::drop_console,
     hoist_decls::DeclHoisterConfig,
@@ -5,9 +6,10 @@ use self::{
 };
 use crate::{
     analyzer::{analyze, ProgramData, UsageAnalyzer},
-    compress::{hoist_decls::decl_hoister, pure::pure_optimizer},
+    compress::hoist_decls::decl_hoister,
     debug::{dump, invoke},
     marks::Marks,
+    mode::Mode,
     option::CompressOptions,
     util::{now, unit::CompileUnit, Optional},
     MAX_PAR_DEPTH,
@@ -41,11 +43,15 @@ mod optimize;
 mod pure;
 mod util;
 
-pub(crate) fn compressor<'a>(
+pub(crate) fn compressor<'a, M>(
     globals: &'a Globals,
     marks: Marks,
     options: &'a CompressOptions,
-) -> impl 'a + JsPass {
+    mode: &'a M,
+) -> impl 'a + JsPass
+where
+    M: Mode,
+{
     let console_remover = Optional {
         enabled: options.drop_console,
         visitor: drop_console(),
@@ -59,12 +65,16 @@ pub(crate) fn compressor<'a>(
         data: None,
         optimizer_state: Default::default(),
         left_parallel_depth: 0,
+        mode,
     };
 
     chain!(console_remover, as_folder(compressor), expr_simplifier())
 }
 
-struct Compressor<'a> {
+struct Compressor<'a, M>
+where
+    M: Mode,
+{
     globals: &'a Globals,
     marks: Marks,
     options: &'a CompressOptions,
@@ -74,15 +84,23 @@ struct Compressor<'a> {
     optimizer_state: OptimizerState,
     /// `0` means we should not create more threads.
     left_parallel_depth: u8,
+
+    mode: &'a M,
 }
 
-impl CompilerPass for Compressor<'_> {
+impl<M> CompilerPass for Compressor<'_, M>
+where
+    M: Mode,
+{
     fn name() -> Cow<'static, str> {
         "compressor".into()
     }
 }
 
-impl Compressor<'_> {
+impl<M> Compressor<'_, M>
+where
+    M: Mode,
+{
     fn handle_stmt_likes<T>(&mut self, stmts: &mut Vec<T>)
     where
         T: StmtLike,
@@ -114,7 +132,7 @@ impl Compressor<'_> {
     /// Optimize a bundle in a parallel.
     fn visit_par<N>(&mut self, nodes: &mut Vec<N>)
     where
-        N: Send + Sync + for<'aa> VisitMutWith<Compressor<'aa>>,
+        N: Send + Sync + for<'aa> VisitMutWith<Compressor<'aa, M>>,
     {
         log::debug!("visit_par(left_depth = {})", self.left_parallel_depth);
 
@@ -129,6 +147,7 @@ impl Compressor<'_> {
                     data: None,
                     optimizer_state: Default::default(),
                     left_parallel_depth: 0,
+                    mode: self.mode,
                 };
                 node.visit_mut_with(&mut v);
 
@@ -148,6 +167,7 @@ impl Compressor<'_> {
                             data: None,
                             optimizer_state: Default::default(),
                             left_parallel_depth: self.left_parallel_depth - 1,
+                            mode: self.mode,
                         };
                         node.visit_mut_with(&mut v);
 
@@ -164,7 +184,7 @@ impl Compressor<'_> {
 
     fn optimize_unit_repeatedly<N>(&mut self, n: &mut N)
     where
-        N: CompileUnit + VisitWith<UsageAnalyzer> + for<'aa> VisitMutWith<Compressor<'aa>>,
+        N: CompileUnit + VisitWith<UsageAnalyzer> + for<'aa> VisitMutWith<Compressor<'aa, M>>,
     {
         if cfg!(feature = "debug") {
             log::debug!(
@@ -210,7 +230,7 @@ impl Compressor<'_> {
     /// Optimize a module. `N` can be [Module] or [FnExpr].
     fn optimize_unit<N>(&mut self, n: &mut N)
     where
-        N: CompileUnit + VisitWith<UsageAnalyzer> + for<'aa> VisitMutWith<Compressor<'aa>>,
+        N: CompileUnit + VisitWith<UsageAnalyzer> + for<'aa> VisitMutWith<Compressor<'aa, M>>,
     {
         self.data = Some(analyze(&*n, Some(self.marks)));
 
@@ -282,7 +302,7 @@ impl Compressor<'_> {
 
             let start_time = now();
 
-            let mut visitor = pure_optimizer(&self.options, self.marks);
+            let mut visitor = pure_optimizer(&self.options, self.marks, self.mode);
             n.apply(&mut visitor);
             self.changed |= visitor.changed();
 
@@ -312,6 +332,7 @@ impl Compressor<'_> {
                 self.options,
                 self.data.as_ref().unwrap(),
                 &mut self.optimizer_state,
+                self.mode,
             );
             n.apply(&mut visitor);
             self.changed |= visitor.changed();
@@ -368,7 +389,10 @@ impl Compressor<'_> {
     }
 }
 
-impl VisitMut for Compressor<'_> {
+impl<M> VisitMut for Compressor<'_, M>
+where
+    M: Mode,
+{
     noop_visit_mut_type!();
 
     fn visit_mut_fn_expr(&mut self, n: &mut FnExpr) {
